"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.S3Helper = void 0;
const AWS = require("aws-sdk");
const typescript_helper_functions_1 = require("typescript-helper-functions");
const signed_url_type_1 = require("./signed-url-type");
/**
 * S3 Helper
 */
class S3Helper extends typescript_helper_functions_1.BaseClass {
    /**
     * Create new instance of S3Helper
     * @param logger {ILogger} Logger
     * @param repository {AWS.S3} A new repository will be created if not supplied
     * @param options {AWS.S3.ClientConfiguration} Only needed if a repository is supplied
     */
    constructor(logger, repository, options) {
        super(logger);
        this.Repository = repository || new AWS.S3(options);
    }
    /**
     * Copy an object from a source to a target
     * @param sourceBucket {string} Source bucket name
     * @param sourceKey {string} Source key
     * @param destinationBucket {string} Destination bucket name
     * @param destinationKey {string} Destination key
     */
    CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.CopyObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { sourceBucket, sourceKey, destinationBucket, destinationKey });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(sourceBucket)) {
                throw new Error(`[${action}]-Must supply sourceBucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(sourceKey)) {
                throw new Error(`[${action}]-Must supply sourceKey`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(destinationBucket)) {
                throw new Error(`[${action}]-Must supply destinationBucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(destinationKey)) {
                throw new Error(`[${action}]-Must supply destinationKey`);
            }
            // create params object
            const params = {
                Bucket: destinationBucket,
                CopySource: `${sourceBucket}/${sourceKey}`,
                Key: destinationKey,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.copyObject(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Create a S3 bucket
     * @param name {string} Bucket name
     */
    CreateBucketAsync(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.CreateBucketAsync.name}`;
            this.LogHelper.LogInputs(action, { name });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(name)) {
                throw new Error(`[${action}]-Must supply name`);
            }
            // create params object
            const params = {
                Bucket: name,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.createBucket(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Delete a S3 bucket
     * @param name {string} Bucket name
     */
    DeleteBucketAsync(name) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteBucketAsync.name}`;
            this.LogHelper.LogInputs(action, { name });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(name)) {
                throw new Error(`[${action}]-Must supply name`);
            }
            // create params object
            const params = {
                Bucket: name,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteBucket(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Delete an object
     * @param bucket {string} Bucket name
     * @param key {string} Object key to delete
     */
    DeleteObjectAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // create params object
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteObject(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Delete multiple objects
     * @param bucket {string} Bucket name
     * @param keys {string[]} Array of object keys to delete
     */
    DeleteObjectsAsync(bucket, keys) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, keys });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (!keys || keys.length === 0) {
                throw new Error(`[${action}]-Must supply at least one key`);
            }
            // create array of ObjectIdentifiers
            const keysArray = [];
            for (const key of keys) {
                keysArray.push({ Key: key });
            }
            // create params object
            const params = {
                Bucket: bucket,
                Delete: { Objects: keysArray },
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.deleteObjects(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Delete all the object tags off of an object
     * @param bucket {string} Bucket name
     * @param key {string} Object key
     */
    DeleteObjectTagsAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.DeleteObjectTagsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.deleteObjectTagging(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Get metadata about a bucket
     * @param bucket {string} Bucket
     */
    GetBucketMetadataAsync(bucket) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetBucketMetadataAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            const params = {
                Bucket: bucket,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.headBucket(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Get a JSON typed object from S3
     * @param bucket {string} Bucket to retrieve from
     * @param key {string} File prefix and name
     */
    GetObjectAsJsonAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = yield this.GetObjectContentsAsync(bucket, key);
            const json = data ? data.toString() : '';
            return JSON.parse(json);
        });
    }
    /**
     * Get an object from S3
     * @param bucket {string} Bucket to retrieve from
     * @param key {string} File prefix and name
     */
    GetObjectAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // create params object
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.getObject(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Get the contents of an S3 object
     * @param bucket {string} Bucket name
     * @param key {string} Object key
     */
    GetObjectContentsAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const data = (yield this.GetObjectAsync(bucket, key)).Body;
            return data ? data : undefined;
        });
    }
    /**
     * Get metadata about an object
     * @param bucket {string} Bucket
     * @param key  {string} Object key
     */
    GetObjectMetadataAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectMetadataAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.headObject(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response.Metadata;
        });
    }
    /**
     * Gets the tags for an object
     * @param bucket {string} Bucket name
     * @param key {string} Object key
     */
    GetObjectTagsAsync(bucket, key) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetObjectTagsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.getObjectTagging(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response.TagSet;
        });
    }
    /**
     * Get a signed url to upload to or download from
     * @param bucket {string} Bucket name
     * @param key {string} Object key
     * @param type {SignedUrlType} Type of signed url to get
     * @param acl {AWS.S3.ObjectCannedACL} ACL of file if uploading
     * @param timeoutInMinutes {number} Timeout for the signed url
     */
    GetSignedUrl(bucket, key, type, timeoutInMinutes = 5, acl) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.GetSignedUrl.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, type, acl, timeoutMinutes: timeoutInMinutes });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // determine upload or download url
            let operation = '';
            switch (type) {
                case signed_url_type_1.SignedUrlType.Download:
                    operation = 'getObject';
                    break;
                case signed_url_type_1.SignedUrlType.Upload:
                    operation = 'putObject';
                default:
                    break;
            }
            const params = {
                Bucket: bucket,
                Key: key,
                Expires: timeoutInMinutes * 60,
                ACL: acl,
            };
            this.LogHelper.LogRequest(action, params);
            const response = yield this.Repository.getSignedUrl(operation, params);
            this.LogHelper.LogResponse(action, { response });
            return response;
        });
    }
    /**
     * Move a file within a bucket or to a different bucket
     * @param sourceBucket {string} Source bucket name
     * @param sourceKey {string} Source object key
     * @param destinationBucket {string} Destination bucket name
     * @param destinationKey {string} Destination object key
     */
    MoveObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey) {
        return __awaiter(this, void 0, void 0, function* () {
            const copyResult = yield this.CopyObjectAsync(sourceBucket, sourceKey, destinationBucket, destinationKey);
            if (!this.ObjectOperations.IsNullOrEmpty(copyResult)) {
                yield this.DeleteObjectAsync(sourceBucket, sourceKey);
            }
        });
    }
    /**
     * Upload a file to S3 Bucket
     * @param bucket {string} Bucket to upload to
     * @param key {string} File prefix and name
     * @param body {string} File contents
     * @param acl {S3CannedACL} S3Canned ACL. Default is 'bucket-owner-full-control'
     * @param encoding {string} File encoding. Default is 'utf-8'
     * @returns Promise<string> - URL of uploaded file
     */
    PutObjectAsync(bucket, key, body, acl, encoding) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.PutObjectAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, body, acl });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            // set defaults
            if (this.ObjectOperations.IsNullOrWhitespace(acl)) {
                acl = 'bucket-owner-full-control';
            }
            if (this.ObjectOperations.IsNullOrWhitespace(encoding)) {
                encoding = 'utf-8';
            }
            // create params object
            const params = {
                ACL: acl,
                Body: body,
                Bucket: bucket,
                ContentEncoding: encoding,
                Key: key,
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.putObject(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
    /**
     * Inserts or updates a tag on an object
     * @param bucket {string} Bucket name
     * @param key {string} Object key
     * @param tagName {string} Tag name
     * @param tagValue {string} Tag value
     */
    SetObjectTagAsync(bucket, key, tagName, tagValue) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.SetObjectTagAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, tagName, tagValue });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(tagName)) {
                throw new Error(`[${action}]-Must supply tagName`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(tagValue)) {
                throw new Error(`[${action}]-Must supply tagValue`);
            }
            // get the original
            const tags = yield this.GetObjectTagsAsync(bucket, key);
            // find if it exists
            const tag = tags.find((t) => t.Key === tagName);
            if (tag) {
                tag.Value = tagValue;
            }
            else {
                tags.push({ Key: tagName, Value: tagValue });
            }
            return this.SetObjectTagsAsync(bucket, key, tags);
        });
    }
    /**
     * Sets the tags on an object
     * @param bucket {string} Bucket name
     * @param key {string} Object key
     * @param tags {AWS.S3.TagSet} Tags
     */
    SetObjectTagsAsync(bucket, key, tags) {
        return __awaiter(this, void 0, void 0, function* () {
            const action = `${S3Helper.name}.${this.SetObjectTagsAsync.name}`;
            this.LogHelper.LogInputs(action, { bucket, key, tags });
            // guard clauses
            if (this.ObjectOperations.IsNullOrWhitespace(bucket)) {
                throw new Error(`[${action}]-Must supply bucket`);
            }
            if (this.ObjectOperations.IsNullOrWhitespace(key)) {
                throw new Error(`[${action}]-Must supply key`);
            }
            if (this.ObjectOperations.IsNullOrEmpty(tags)) {
                throw new Error(`[${action}]-Must supply tags`);
            }
            const params = {
                Bucket: bucket,
                Key: key,
                Tagging: { TagSet: tags },
            };
            this.LogHelper.LogRequest(action, params);
            // make AWS call
            const response = yield this.Repository.putObjectTagging(params).promise();
            this.LogHelper.LogResponse(action, response);
            return response;
        });
    }
}
exports.S3Helper = S3Helper;
//# sourceMappingURL=helper.js.map